
  //  a recursive descent parser, 
  //  written test-first

#include "recursiveDescentParser.h"
#include "test.h"


bool TestCase::all_tests_passed = true;
TestCase::TestCases_t TestCase::cases;


using namespace parser;

#define ASSERT_BEEF(function, beef)   \
        CHECK_EQUAL(false, function.getValid());   \
        CHECK_EQUAL(beef, function.m_beef);


TEST_(TestCase, term)
{
    CHECK_CLOSE( 3.0, parse("3") );
    CHECK_CLOSE( 2.0, parse("2") );
    CHECK_CLOSE( 2.8, parse("2.8") );
    CHECK_CLOSE( 2.8, parse("  2.8") );
    CHECK_CLOSE(-2.8, parse("-2.8") );
    ASSERT_BEEF( parse(".q")   , "syntax error" );
    ASSERT_BEEF( parse("- 1.0"), "syntax error" );

}


TEST_(TestCase, sum)
{
    CHECK_CLOSE( 5.0, parse("3 + 2") );
    CHECK_CLOSE( 1.0, parse("3 + -2") );
    CHECK_CLOSE(11.0, parse("3 + 8") );
    CHECK_CLOSE(11.0, parse("3 + 4 + 4") );
    CHECK_CLOSE( 5.0, parse("-3 + 4 + 4") );
    ASSERT_BEEF( parse("3 + .q")  , "syntax error" );
    ASSERT_BEEF( parse("3 + 12 +"), "syntax error" );
}


TEST_(TestCase, subtraction)
{
    CHECK_CLOSE(  1.0, parse("3 - 2") );
    CHECK_CLOSE(  5.0, parse("3 - -2") );
    CHECK_CLOSE( -5.0, parse("3 - 8") );
    CHECK_CLOSE( -5.0, parse("3 - 4 - 4") );
    CHECK_CLOSE(-11.0, parse("-3 - 4 - 4") );
    CHECK_CLOSE( -3.0, parse("-3 + 4 - 4") );
    CHECK_CLOSE( -3.0, parse("-3 - 4 + 4") );
    ASSERT_BEEF( parse("3 - .q")  , "syntax error" );
    ASSERT_BEEF( parse("3 + 12 -"), "syntax error" );
}


TEST_(TestCase, multiplication)
{
    CHECK_CLOSE(  6.0, parse("3 * 2") );
    CHECK_CLOSE( -6.0, parse("3 * -2") );
    CHECK_CLOSE( 24.0, parse("3 * 8") );
    CHECK_CLOSE(  8.0, parse("3 * 4 - 4") );
    CHECK_CLOSE(-19.0, parse("-3 - 4 * 4") );
    ASSERT_BEEF( parse("3 * .q")  , "syntax error" );
    ASSERT_BEEF( parse("3 - 12 *"), "syntax error" );
}


TEST_(TestCase, division)
{
    CHECK_CLOSE(  1.5  , parse("3 / 2") );
    CHECK_CLOSE( -1.5  , parse("3 / -2") );
    CHECK_CLOSE(  0.375, parse("3 / 8") );
    CHECK_CLOSE( -3.25 , parse("3 / 4 - 4") );
    CHECK_CLOSE( -3.0  , parse("-3 / 4 * 4") );
    ASSERT_BEEF( parse("3 / .q")  , "syntax error" );
    ASSERT_BEEF( parse("3 + 12 /"), "syntax error" );
}


TEST_(TestCase, parens)
{
    CHECK_CLOSE(  1.5  , parse("(3) / 2") );
    CHECK_CLOSE( -1.5  , parse("(3 / -2)") );
    CHECK_CLOSE(  0.375, parse("3 / (8)") );
    CHECK_CLOSE( -3.0  , parse("3.0 / (4-5)") );
    CHECK_CLOSE( -3.0  , parse("(-3.0 / 4) * 4") );
    CHECK_CLOSE( -3.0  , parse("(-3.0 / 4)*4") );
    CHECK_CLOSE( -3.0  , parse("3.0 / ((((4 - 5) )) )") );   
    ASSERT_BEEF( parse("3.0 / (4 - 4)") , "division by zero error" );
}
    
/*TEST_(TestCase, brokeParens)
{
    Fallible_double_t beef = parse("3 + 12)");
    ASSERT_BEEF( beef, "incomplete expression" );
}*/

TEST_(TestCase, brokeParensToo)
{
//    ASSERT_BEEF( parse("(3 / .q"), "syntax error" );
    //
    ASSERT_BEEF( parse("("), "syntax error" );

    ASSERT_BEEF( parse("5 + ("), "syntax error" );

}


TEST_(TestCase, exponentiation)
{
    CHECK_CLOSE(   9.0     , parse("3 ^ 2") );
    CHECK_CLOSE(   0.111111, parse("3 ^ -2") );
    CHECK_CLOSE(6561.0     , parse("3 ^ 8") );
    CHECK_CLOSE(  77.0     , parse("3 ^ 4 - 4") );
    CHECK_CLOSE( 324.0     , parse("-3 ^ 4 * 4") );
    CHECK_CLOSE(   9.0     , parse("(3) ^ 2") );
    CHECK_CLOSE(   0.111111, parse("(3^-2)") );
    CHECK_CLOSE(6561.0     , parse("3^(8)") );
}


TEST_(TestCase, exponentiationAndParens)
{
    CHECK_CLOSE(   0.333333, parse("3.0 ^(4 - 5)") );
    CHECK_CLOSE(   1.0     , parse("3.0 ^ (4-4)") );
    CHECK_CLOSE( 324.0     , parse("(-3.0 ^ 4) * 4") );
    CHECK_CLOSE(   0.333333, parse("3.0 ^ ((((4 - 5) )) )") );   
    ASSERT_BEEF( parse("3 ^ .q")  , "syntax error" );
    ASSERT_BEEF( parse("3 + 12 ^"), "syntax error" );
}


    bool
runTests()
{
    bool result = TestCase::runTests();
    char const * determination = "";
    
    if (result)
        determination = "All tests passed!\n";
    else
        determination = "Tests failed...\n";

    cout << determination;
    OutputDebugString("\n");
    OutputDebugString(determination);
    OutputDebugString("\n");

    return result;
}


int main() {  return ! runTests();  }


