
// a NanoCppUnit in one header...

#ifndef TEST_

#   include <list>
#   include <iostream>
#   include <sstream>
#   define WIN32_LEAN_AND_MEAN
#   include <windows.h>
#   include <math.h>

    using std::cout;
    using std::endl;
    using std::stringstream;

#ifndef _CrtDbgBreak  //  Note this assumes all _CrtDbgBreaks be macroidal
#   define _CrtDbgBreak() __asm { int 3 }
#endif

    class
TestCase
{
  public:
    struct Creator;  //  TODO  this should be private
    typedef std::list<Creator *> TestCases_t;
    TestCases_t static cases;  //  TODO  make me private

    virtual void setUp() {}
    virtual void runCase() = 0;
    virtual void tearDown() {}
    static bool  runTests();
    virtual     ~TestCase() {};
    
  protected:
    static bool all_tests_passed;
};


    struct
TestCase::Creator
{
    typedef std::auto_ptr<TestCase> TestCasePtr_t;
    virtual TestCasePtr_t create() = 0;
    Creator()  {  TestCase::cases.push_back(this);  }
};  //  TODO TFUI this constructs the test case at test time, not before main()


    inline bool
TestCase::runTests()
{
    TestCase::TestCases_t::iterator it(TestCase::cases.begin());
    
    for ( ;  it != TestCase::cases.end();  ++it )
        {
        Creator & aCreator = **it;
        using Creator::TestCasePtr_t;
        TestCasePtr_t pCase (aCreator.create());
        pCase->setUp();
        pCase->runCase();
        pCase->tearDown();  //  TODO  should this call from a destructor?
        }
    return TestCase::all_tests_passed;
}


    template<class target>
    class  //  you can only call create() thru base
TestCaseCreatorImplementation:  public TestCase::Creator
{
    virtual TestCasePtr_t create() {  return TestCasePtr_t(new target);  };
};


#define TEST_(suite, target)             \
    struct suite##target:  virtual suite \
    { void runCase(); };     \
    TestCaseCreatorImplementation<suite##target> a##suite##target##Implementation;    \
    void suite##target::runCase()

#define CHECK_EQUAL(sample, result)          \
    if ((sample) != (result))  {  stringstream out;   \
        out << __FILE__ << "(" << __LINE__ << ") : "; \
        out << #sample << "(" << (sample) << ") should equal "; \
        out << #result << "(" << (result) << ")";     \
        cout << out.str() << endl;                    \
        OutputDebugStringA(out.str().c_str());        \
        OutputDebugStringA("\n");                     \
        all_tests_passed = false;                     \
        _CrtDbgBreak();  } 
        
#define CHECK_NOT_EQUAL(sample, result)          \
    if ((sample) == (result))  {  stringstream out;   \
        out << __FILE__ << "(" << __LINE__ << ") : "; \
        out << #sample << "(" << (sample) << ") should not equal "; \
        out << #result << "(" << (result) << ")";     \
        cout << out.str() << endl;                    \
        OutputDebugStringA(out.str().c_str());        \
        OutputDebugStringA("\n");                     \
        all_tests_passed = false;                     \
        _CrtDbgBreak();  } 
        
#define CHECK(sample)          \
    if (!(sample))  {  stringstream out;   \
        out << __FILE__ << "(" << __LINE__ << ") : "; \
        out << "!" #sample << "(" << (sample) << ")"; \
        cout << out.str() << endl;                    \
        OutputDebugStringA(out.str().c_str());        \
        OutputDebugStringA("\n");                     \
        all_tests_passed = false;                     \
        _CrtDbgBreak();  } 

inline bool tolerance(double Subject, double Reference)
{
    return fabs(Subject - Reference) < (0.001 + fabs(Subject)) / 100;
}

#define CHECK_CLOSE(sample, result)          \
    if (!tolerance(sample, result))  {  stringstream out;   \
        out << __FILE__ << "(" << __LINE__ << ") : "; \
        out << #sample << "(" << (sample) << ") != "; \
        out << #result << "(" << (result) << ")";     \
        cout << out.str() << endl;                    \
        OutputDebugStringA(out.str().c_str());        \
        OutputDebugStringA("\n");                     \
        all_tests_passed = false;                     \
        _CrtDbgBreak();  } 


#endif